
/* -*- MaTX -*-
 *
 *  Ric()  Solve a Riccati equation
 *
 *    P = Ric(A, Q, R);
 *
 *  computes the Riccati equation "A#*P + P*A + P*R*P = Q"
 *
 *  To calculate the solution of  "A#*P + P*A - P*R*P + Q = 0"
 *  call ths function as  "P = Ric(A, -Q, -R)"
 *
 *  Synopsis:  P = Ric(A, Q, R)
 *               Matrix P;
 *               Matrix A,Q,R;
 */

Func Matrix Ric(A, Q, R)
	Matrix A;
	Matrix Q;
	Matrix R;
{
	Matrix P, H, U, V, T, Perr;
	Real res, eps, tmp;

	eps = 1.0E-6;

	H = [[ A,  R]
		 [ Q, -A#]];

	T = eigvec(H);
	U = T(0, 1, A);
	V = T(1, 1, A);
	P = V * U~;

	if (isreal(A) && isreal(Q) && isreal(R)) {
		P = Re(P);
		P = (P' + P)/2;
	} else {
		P + (P# + P)/2;
	}

	// Check redidual
	Perr = A#*P + P*A + P*R*P - Q;
	res = frobnorm(Perr);
	if (res > 1.0) {
		warning("Warning: Frobenius norm of the Riccati residual: %g\n", res);
	}

	// Check jw-axis roots:
	tmp = min(abs(Array(Re(eigval(A + R*P)))));
	if (tmp < eps) {
		warning("\nWarning: There are jw-axis closed loop poles at (%g, *)  ....\n", -tmp);
		warning("         Results may be incorrect !!\n");
	}

	// Check positive definiteness of P:
	tmp = min(Re(eigval(P)));
	if (tmp < 0.0) {
		warning("\nWarning: P is not positive ....\n");
		warning("The smallest eigenvalue of P is %g\n", tmp);
	}

	if (isreal(A) && isreal(Q) && isreal(R)) {
		return Re(P);
	} else {
		return P;
	}
}
